@@ -51,6 +51,10 @@ cairo_surface_t *ev_document_misc_surface_rotate_and_scale (cairo_surface_t *sur
 							    gint             dest_width,
 							    gint             dest_height,
 							    gint             dest_rotation);
+void             ev_document_misc_invert_surface (cairo_surface_t *surface);
+void		 ev_document_misc_invert_pixbuf  (GdkPixbuf       *pixbuf);
+
+gdouble          ev_document_misc_get_screen_dpi (GdkScreen *screen);
 
 G_END_DECLS
 
